@@ -422,3 +422,17 @@ if($err) {
 	warn $err;
 	exit(1);
 }
+
+__END__
+
+=encoding utf8
+
+=head1 NAME
+
+xyzzy - process manager for Xyzzy applications
+
+=head1 SYNOPSIS
+
+ xyzzy /path/to/app.cfg
+
+=cut
