@@ -99,20 +99,35 @@ sub xyzzy_request_response {
 	$options{PATH_INFO} //= $path;
 	$options{QUERY_STRING} //= $query // '';
 
+	my $cfg = $handler
+		if UNIVERSAL::isa($handler, 'Xyzzy');
+
 	$handler = xyzzy_from_file($handler)
 		unless ref $handler;
 
+	$cfg = $handler
+		if UNIVERSAL::isa($handler, 'Xyzzy');
+
 	$handler = xyzzy_from_config(@$handler)
 		unless blessed $handler;
 
+	$cfg = $handler
+		if UNIVERSAL::isa($handler, 'Xyzzy');
+
 	$handler = $handler->handler
 		if UNIVERSAL::isa($handler, 'Xyzzy');
 
+	$cfg = $handler
+		if UNIVERSAL::isa($handler, 'Xyzzy');
+
+	$cfg = $handler->cfg
+		unless defined $cfg;
+
 	my ($inbuf, $outbuf, $errbuf) = ($options{REQUEST_BODY} // '', '', '');
 	open(my $in, '<', \$inbuf) or die;
 	open(my $out, '>', \$outbuf) or die;
 	open(my $err, '>', \$errbuf) or die;
-	my $req = new Xyzzy::Request::Root(in => $in, out => $out, err => $err, env => {%$default_env, %options});
+	my $req = new Xyzzy::Request::Root(cfg => $cfg, in => $in, out => $out, err => $err, env => {%$default_env, %options});
 	my $res = eval { $handler->handle($req) };
 	unless($res) {
 		my $err = $@;
@@ -129,17 +144,16 @@ sub xyzzy_request_response {
 sub xyzzy_request {
 	my $res = xyzzy_request_response(@_);
 	my $content = $res->content;
-	confess("Xyzzy::Response content must be binary! Convert logical characters explicitly using Encode::encode or Encode::encode_utf8")
-		if utf8::is_utf8($content);
+	utf8::downgrade($content);
 	return $content;
 }
 
 sub xyzzy_request_as_cgi {
 	my $res = xyzzy_request_response(@_);
 	my $as_cgi = $res->as_cgi;
-	confess("Xyzzy::Response content and headers must be binary! Convert logical characters explicitly using Encode::encode or Encode::encode_utf8")
-		if utf8::is_utf8($as_cgi);
-	return join('', @$as_cgi);
+	$as_cgi = join('', @$as_cgi);
+	utf8::downgrade($as_cgi);
+	return $as_cgi;
 }
 
 1;
