@@ -22,7 +22,7 @@ field cas_validate_url => sub { shift->cas_base_url . '/validate' };
 field cas_domain => undef;
 field cas_force_domain => undef;
 field cas_force_https => !undef;
-field cas_use_curl => !undef;
+field cas_use_curl => $have_curl;
 
 field session_timeout => 36000;
 field request_timeout => 3600;
