@@ -128,9 +128,13 @@ field cur_session => sub {
 
 sub cas_redirect {
 	my $url = shift;
-	my $status = new Xyzzy::Status(302);
+	# The client must not be tempted to direct POST/PUT/DELETE/etc
+	# requests at the CAS server.
+	my $code = $self->request_method =~ /^(?:GET|HEAD)$/a ? 302 : 403;
+	my $status = new Xyzzy::Status($code);
 	my $res = $status->response;
-	$res->addheader(Location => $url);
+	$res->addheader(Location => $url)
+		if $code == 302;
 	die $res;
 }
 
