@@ -4,12 +4,12 @@ use warnings FATAL => 'all';
 
 package Xyzzy::Crypto;
 
-use MIME::Base64;
+use MIME::Base64 qw(encode_base64url decode_base64url);
 use Encode;
 use Digest::SHA qw(hmac_sha256);
 use Crypt::OpenSSL::Random;
 use Time::HiRes qw(time);
-use Xyzzy::Util qw(utf8_testandset_inplace utf8_disable);
+use Xyzzy::Util qw(utf8_testandset utf8_disable);
 
 use Clarity -self;
 
@@ -25,12 +25,12 @@ sub random_bytes {
 
 sub random_hex {
 	my $hex = unpack('H*', $self->random_bytes($@));
-	$hex =~ tr/a-z/A-Z/;
+	$hex =~ tr/a-f/A-F/;
 	return $hex;
 }
 
 sub hmac {
-	return hmac_sha256(map { utf8_disable($_) } @_, $self->secret);
+	return hmac_sha256(utf8_disable(@_), $self->secret);
 }
 
 sub create_token_data {
@@ -56,11 +56,7 @@ sub create_token_data {
 sub create_token {
 	my ($time, $salt, $raw) = $self->create_token_data(@_);
 
-	my $encoded = encode_base64($raw, '');
-	$encoded =~ s/=+$//;
-	$encoded =~ tr|/+|._|;
-
-	return $time, $salt, $encoded;
+	return $time, $salt, encode_base64url($raw);
 }
 
 sub check_token_data {
@@ -88,7 +84,7 @@ sub check_token_data {
 	die "Expired token\n"
 		if $expiry && $time + $expiry < $now;
 
-	utf8_testandset_inplace(@keys);
+	utf8_testandset(@keys);
 
 	if(wantarray) {
 		return $time, $salt, @keys;
@@ -104,11 +100,9 @@ sub check_token {
 		unless defined $token;
 
 	die "Invalid token\n"
-		unless $token =~ /^[a-zA-Z0-9_.]+$/;
+		unless $token =~ /^[a-zA-Z0-9_-]+$/a;
 
-	$token =~ tr|._|/+|;
-	$token .= '=' x -(length($token) % -4);
-	$token = decode_base64($token);
+	$token = decode_base64url($token);
 
 	return $self->check_token_data($hidden, $token, $expiry);
 }
@@ -132,7 +126,7 @@ Tokens are signed using a passphrase that can be configured using
 Xyzzy::Crypto::Config.
 
 Ascii encoded tokens use a base64 alphabet using A-Za-z0-9 and the
-characters '.' and '-'.
+characters '-' and '_'.
 
 =head2 random_bytes($num)
 
