@@ -10,14 +10,17 @@ use Xyzzy::Util qw(parse_time);
 use Xyzzy -self;
 
 field cryptosecret => sub { die "no CryptoSecret configured\n" };
-sub set_cryptosecret {
-	my $key = shift;
-	my $len = int(length($key)*2/3);
-	my @values = (substr($key, 0, $len), substr($key, -$len));
-	$self->cryptosecret([map { sha256($_) } @values]);
-}
+*set_cryptosecret = *cryptosecret;
 
 field cryptoclockjitter => 0;
 sub set_cryptoclockjitter {
 	$self->cryptoclockjitter(parse_time(shift));
 }
+
+# 512 bits should be enough for anyone
+field cryptoseedbytes => 64;
+*set_cryptoseedbytes = *cryptoseedbytes;
+
+# http://www.2uo.de/myths-about-urandom/
+field cryptoseedsource => '/dev/urandom';
+*set_cryptoseedsource = *cryptoseedsource;
