@@ -23,5 +23,6 @@ field db_options => {
 	RaiseError => 1,
 	AutoCommit => 0,
 	ShowErrorStatement => 1,
-	pg_enable_utf8 => 1
+	FetchHashKeyName => 'NAME_lc',
+	pg_enable_utf8 => 1,
 };
