@@ -11,7 +11,10 @@ sub serialize {
 
 	if($self->req->msie) {
 		$res->mimetype('text/html', charset => 'UTF-8');
-		$res->content($self->toStringHTML);
+		my $html = $self->toStringHTML;
+		# elements marked F in http://www.w3.org/TR/html4/index/elements.html:
+		$html =~ s{<((area|base(?:font)?|br|col|frame|hr|img|input|isindex|link|meta|param)(?: [^"'=<>]+(?:=(?:"[^"]*"|'[^']*'))?)*)></\2>}{<$1 />}gi;
+		$res->content($html);
 	} else {
 		$res->mimetype('application/xhtml+xml');
 		$res->content($self->toString);
