@@ -29,34 +29,74 @@ sub iso8601_strict {
 		$year + 1900, $mon + 1, $mday, $hour, $min, $sec)
 }
 
+# upgrade to UTF-8 if possible, without changing any bytes (deprecated)
 sub utf8_testandset_inplace {
 	foreach my $val (@_) {
-		# upgrade to UTF-8 if possible, without changing any bytes
+		next unless defined $val;
 		Encode::_utf8_on($val);
 		Encode::_utf8_off($val) unless utf8::valid($val);
 	}
 	return;
 }
 
+# upgrade to UTF-8 if possible, without changing any bytes
 sub utf8_testandset {
-	my $str = shift;
-	return undef unless defined $str;
-	utf8_testandset_inplace($str);
-	return $str;
+	my $wantlist = wantarray;
+	if($wantlist) { # list context
+		return map {
+			my $val = $_; # make sure we work on a copy
+			if(defined $val) {
+				Encode::_utf8_on($val);
+				Encode::_utf8_off($val) unless utf8::valid($val);
+			}
+			$val
+		} @_;
+	} elsif(defined $wantlist) { # scalar context
+		my $val = pop;
+		if(defined $val) {
+			Encode::_utf8_on($val);
+			Encode::_utf8_off($val) unless utf8::valid($val);
+		}
+		return $val;
+	} else { # void context
+		foreach my $val (@_) {
+			if(defined $val) {
+				Encode::_utf8_on($val);
+				Encode::_utf8_off($val) unless utf8::valid($val);
+			}
+		}
+	}
+	return;
 }
 
+# undo utf8_testandset_inplace (deprecated)
 sub utf8_disable_inplace {
 	foreach my $val (@_) {
+		next unless defined $val;
 		Encode::_utf8_off($val);
 	}
 	return;
 }
 
+# undo utf8_testandset
 sub utf8_disable {
-	my $str = shift;
-	return undef unless defined $str;
-	Encode::_utf8_off($str);
-	return $str;
+	my $wantlist = wantarray;
+	if($wantlist) { # list context
+		return map {
+			my $val = $_; # make sure we work on a copy
+			Encode::_utf8_off($val) if defined $val;
+			$val
+		} @_;
+	} elsif(defined $wantlist) { # scalar context
+		my $val = pop;
+		Encode::_utf8_off($val) if defined $val;
+		return $val;
+	} else { # void context
+		foreach my $val (@_) {
+			Encode::_utf8_off($val) if defined $val;
+		}
+	}
+	return;
 }
 
 sub uri_escape_auto {
